///////////////////////////////////////////////////////////////////////////////
//
// "libRTEA" , rev 0.2
//
//   : a light Rageworx's TEA encrypt/decrypt tool for ANSI-C++
// ===========================================================================
//
// (C) Copyright 2011 ,  rageworx@gamil.com
//
///////////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>

#include "libRTEA.h"

#define  SUM_INIT       0xC6EF3720
#define  DELTA          0x9E3779B9
#define  PAD_BYTE       0xA5
#define  BQUEUE_SIZE    sizeof(unsigned int) * 2

void RTEAcodec::Hex2Char(char const* aChar, unsigned char *rHex)
{
    int     cnt;

    *rHex = 0;

    if(strlen(aChar) < 2)
        return;

    for (cnt=0; cnt<2; cnt++)
    {
        if ( *(aChar + cnt) >= '0' && *(aChar + cnt) <= '9' )
        {
            *rHex = (*rHex << 4) + (*(aChar + cnt) - '0');
        }
        else
        if ( *(aChar + cnt) >= 'A' && *(aChar + cnt) <= 'F' )
        {
            *rHex = (*rHex << 4) + (*(aChar + cnt) - 'A' + 10);
        }
        else
        if ( *(aChar + cnt) >= 'a' && *(aChar + cnt) <= 'f' )
        {
            *rHex = (*rHex << 4) + (*(aChar + cnt) - 'a' + 10);
        }
        else
            break;
    }
}

void RTEAcodec::encipher(const unsigned long *v,unsigned long *w, const unsigned long *k)
{
    register unsigned long  y     = v[0];
    register unsigned long  z     = v[1];
    register unsigned long  sum   = 0;
    register unsigned long  delta = DELTA;
    register unsigned long  n     = 32;

    while(n-- > 0)
    {
        y += (z << 4 ^ z >> 5) + z ^ sum + k[sum&3];
        sum += delta;
        z += (y << 4 ^ y >> 5) + y ^ sum + k[sum>>11 & 3];
    }

    w[0]=y;
    w[1]=z;
}

void RTEAcodec::decipher(const unsigned long *v,unsigned long *w, const unsigned long *k)
{
    register unsigned long y     = v[0];
    register unsigned long z     = v[1];
    register unsigned long sum   = SUM_INIT;
    register unsigned long delta = DELTA;
    register unsigned long n     = 32;

   /* sum = delta<<5, in general sum = delta * n */

    while(n-- > 0)
    {
        z -= (y << 4 ^ y >> 5) + y ^ sum + k[sum>>11 & 3];
        sum -= delta;
        y -= (z << 4 ^ z >> 5) + z ^ sum + k[sum&3];
    }

    w[0]=y;
    w[1]=z;
}

void RTEAcodec::CharToHexChar(char* pSrc, int nLenSrc, char* pDst, int nLenDst)
{
    // check NULL and length .
    if((pSrc == NULL) || (nLenSrc == 0) || (pDst == NULL) || (nLenDst == 0))
        return;

    // check length : nLenDst must be bigger than nLenSrc as double of nLenSrc !
    if(nLenDst < (nLenSrc*2))
        return;

    int     cnt;
    char    strTmp[3] = {0};

    for(cnt=0; cnt<nLenSrc; cnt++)
    {
        memset(strTmp,3,0);
        sprintf(strTmp,"%02X",(unsigned char)pSrc[cnt]);
        memcpy(&pDst[cnt*2],strTmp,2);
    }

}

void RTEAcodec::HexCharToChar(char *pSrc, int nLenSrc, char *pDst, int nLenDst)
{
    // check NULL and length .
    if((pSrc == NULL) || (nLenSrc == 0) || (pDst == NULL) || (nLenDst == 0))
        return;

    // check length : nLenDst must be bigger than nLenSrc as half of nLenSrc !
    if(nLenDst < (nLenSrc/2))
        return;

    int             cnt;
    char            strConv[2] = {0};
    unsigned char   aChar;

    for(cnt=0; cnt<(nLenSrc/2); cnt++)
    {
        strConv[0] = pSrc[(cnt*2) + 0];
        strConv[1] = pSrc[(cnt*2) + 1];

        Hex2Char(strConv,&aChar);

        pDst[cnt] = aChar;
    }
}

void RTEAcodec::EncodeBuffer(char* BuffSrc, char* BuffDst, int BufferLen, teaKeyString* Key)
{
    int nNewBufLen = ( BufferLen + 1 ) / 2;
    int buffQue    = 0;

    if((BuffSrc == NULL) || (BuffDst == NULL) || (BufferLen == 0) || (Key == NULL))
        return;


    // check key padding.
    for(int cnt=0; cnt<MAX_KEY_LENGTH+1; cnt++)
    {
        if(*Key[cnt] == 0)
            *Key[cnt] = PAD_BYTE;
    }

    while(nNewBufLen > 0)
    {
        encipher((const unsigned long*)&BuffSrc[buffQue],
                 (unsigned long*)&BuffDst[buffQue],
                 (const unsigned long*)Key);
        buffQue += BQUEUE_SIZE;
        nNewBufLen--;
    }
}

void RTEAcodec::Decodebuffer(char* BuffSrc, char* BuffDst, int BufferLen, teaKeyString* Key)
{
    int nNewBufLen = ( BufferLen + 1 ) / 2;
    int buffQue    = 0;

    if((BuffSrc == NULL) || (BuffDst == NULL) || (BufferLen == 0) || (Key == NULL))
        return;

    // check key padding.
    for(int cnt=0; cnt<MAX_KEY_LENGTH+1; cnt++)
    {
        if(*Key[cnt] == 0)
            *Key[cnt] = PAD_BYTE;
    }

    while(nNewBufLen > 0)
    {
        decipher((const unsigned long*)&BuffSrc[buffQue],
                 (unsigned long*)&BuffDst[buffQue],
                 (const unsigned long*)Key);
        buffQue += BQUEUE_SIZE;
        nNewBufLen--;
    }
}
